
/* 
 * Name: test.c
 * Project: arm-sensor
 * Author: kawamoto@devdrv.co.jp
 * Creation Date: 2009-12-01
 */

#include <usb.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>

#define VENDOR_ID  0x1384
#define PRODUCT_ID 0x0200

#define MAX_DATA_SIZE   5
#define MAX_PACKET_SIZE 8

#define HDR_NULL         ((0 << 3) | 0)	/* ID = 0, LEN = 0 */
#define HDR_ACCELERATION ((1 << 3) | 3)	/* ID = 1, LEN = 3 */
#define HDR_ILLUMINANCE  ((2 << 3) | 1)	/* ID = 2, LEN = 1 */
#define HDR_SWITCH       ((3 << 3) | 1)	/* ID = 3, LEN = 1 */

static unsigned char data[MAX_DATA_SIZE];
static unsigned char buffer[MAX_PACKET_SIZE];
static int main_loop;

void ctrlc(int sig)
{
	main_loop = 0;
	return;
}

int main(int argc, char *argv[])
{
	struct usb_bus *usb_bus;
	struct usb_device *dev;
	struct usb_dev_handle *hdl;
	int ret, i;

	usb_init();
	usb_find_busses();
	usb_find_devices();

	for(usb_bus = usb_get_busses(); usb_bus; usb_bus = usb_bus->next)
	{
		for(dev = usb_bus->devices; dev; dev = dev->next)
		{
			if((dev->descriptor.idVendor == VENDOR_ID)
					&& (dev->descriptor.idProduct == PRODUCT_ID))
			{
				goto find_exit;
			}
		}
	}

	fprintf(stderr, "device not found.\n");
	exit(1);

find_exit:;

	if((hdl = usb_open(dev)) == NULL)
	{
		fprintf(stderr, "usb_open failed.\n");
		exit(1);
	}

#if 0
	if(usb_set_configuration(hdl, dev->config->bConfigurationValue) < 0)
	{
		if(usb_detach_kernel_driver_np(hdl,
				dev->config->interface->altsetting->bInterfaceNumber) < 0)
		{
			fprintf(stderr, "usb_detach_kernel_driver_np failed.\n");
			usb_close(hdl);
			exit(1);
		}
		else
		{
#endif
			if(usb_set_configuration(hdl, dev->config->bConfigurationValue) < 0)
			{
				fprintf(stderr, "usb_set_configuration failed.\n");
				usb_close(hdl);
				exit(1);
			}
#if 0
		}
	}
#endif

#if 0
	if(usb_claim_interface(hdl, dev->config->interface->altsetting->bInterfaceNumber) < 0)
	{
		if(usb_detach_kernel_driver_np(hdl,
				dev->config->interface->altsetting->bInterfaceNumber) < 0)
		{
			fprintf(stderr, "usb_detach_kernel_driver_np failed.\n");
			usb_close(hdl);
			exit(1);
		}
		else
		{
#endif
			if(usb_claim_interface(hdl, dev->config->interface->altsetting->bInterfaceNumber) < 0)
			{
				fprintf(stderr, "usb_claim_interface failed.\n");
				usb_close(hdl);
				exit(1);
			}
#if 0
		}
	}
#endif

	signal(SIGINT, ctrlc);

	main_loop = 1;
	while(main_loop)
	{
		if(usb_interrupt_read(hdl, dev->config->interface->altsetting->endpoint->bEndpointAddress,
				buffer, sizeof(buffer), 2000) < 0)
/*		if(usb_control_msg(hdl, USB_TYPE_CLASS | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
				0x01, 3 << 8, 0, buffer, sizeof(buffer), 2000) < 0)	*/
		{
			fprintf(stderr, "usb_interrupt_read failed.\n");
			usb_release_interface(hdl, 0);
			usb_close(hdl);
			exit(1);
		}
		
		if(buffer[0] == HDR_ACCELERATION)
		{
			data[0] = buffer[1];
			data[1] = buffer[2];
			data[2] = buffer[3];
			
			printf("* ");
			for(i = 0; i < sizeof(data); i++) printf("%3d ", data[i]);
			printf("\n");
		}
		else if(buffer[0] == HDR_ILLUMINANCE)
		{
			data[3] = buffer[1];
			
			printf("* ");
			for(i = 0; i < sizeof(data); i++) printf("%3d ", data[i]);
			printf("\n");
		}
		else if(buffer[0] == HDR_SWITCH)
		{
			data[4] = buffer[1];
			
			printf("* ");
			for(i = 0; i < sizeof(data); i++) printf("%3d ", data[i]);
			printf("\n");
		}
		else
		{
			printf("  ");
			for(i = 0; i < sizeof(data); i++) printf("%3d ", data[i]);
			printf("\n");
		}
	}

	printf("done!\n");

	usb_release_interface(hdl, dev->config->interface->altsetting->bInterfaceNumber);
	usb_close(hdl);

	exit(0);
}

